# User configuration
runs<-c('wrfld01', 'i5eoffb51', 'i5ewpblconv')
cols<-c('black', 'red', 'darkgreen')
range<-192
n.ens<-10
fld<-'V10'
results.dir<-'../results'

# Utility functions
library(boot)
data.manip <- function(d.raw, fld.mult, fld.type){
    d.r <- d.raw[, order(ncol(d.raw):1)] * fld.mult
    if (fld.type == 'min'){
        return(apply(d.r, 1, min))
    } else if (fld.type == 'max'){
        return(apply(d.r, 1, max))
    } else if (fld.type == 'mean'){
        return(apply(d.r, 1, mean))
    } else {
        stop(paste("Unknown field type:",fld.type))
    }
}

# Field-specific configuration
fld.mult <- 1
fld.min <- -Inf
fld.leg.pos <- 'bottomleft'
if (fld == 'P0'){
    fld.name <- 'Minimum Central Pressure'
    fld.range <- c(890,1008)
    fld.leg.pos <- 'bottomleft'
    if (sum(grep('late', runs)) > 0){fld.leg.pos <-'bottomright'}
    fld.units <- bquote(.(fld.name) ~ "(hPa)")
    fld.type <- 'min'
} else if (fld == 'V10'){
    fld.name <- 'Maximum Wind'
    fld.range <- c(0,75)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'max'
    fld.leg.pos <- 'topleft'
    fld.min <- 0
} else if (fld == 'V10R'){
    fld.name <- 'Maximum Inflow Wind'
    fld.range <- c(-20,20)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'min'
} else if (fld == 'V10T'){
    fld.name <- 'Maximum Tangential Wind'
    fld.range <- c(0,100)
    fld.units <- bquote(.(fld.name) ~ "(" * m~s^-1 * ")")
    fld.type <- 'max'
    fld.leg.pos <- 'topleft'
    fld.min <- 0
} else if (fld == 'RT'){
    fld.name <- 'Precipitation Rate'
    fld.mult <- 1000*3600
    fld.range <- c(0,200)
    fld.units <- bquote(.(fld.name) ~ "(" * mm~h^-1 * ")")
    fld.type <- 'max'
} else {
    stop(paste("Error: Unknown field fld =",fld))
}

# Retrieve coordinate vectors
coord.rad<-scan(paste0(results.dir,'/',runs[1],'/coord_rad.dat'))
coord.rad.r<-rev(coord.rad)

# Plot setup
pdf(paste(paste(runs,collapse='-'),'_',fld,'_series.pdf',sep=''), paper='special',
           width=7, height=5.5, bg='white')
default <- par(mar=c(5,6,4,2))
plot(1, 1, type='n', xlim=c(0,range), ylim=fld.range, xaxt='n', yaxt='n',
     xlab='Lead Time (h)', ylab=NA, cex.lab=1.4,
     main=fld.name, cex.main=1.6)
axis(1, at=seq(0, range, by=24),cex.axis=1.4)
axis(2, at=pretty(fld.range), cex.axis=1.4, las=2)
mtext(fld.units, 2, cex=1.4, line=3.7)

# Plot data for all requested runs
col.i <- 0
for (run in runs){
    col.i <- col.i+1

    # Retrieve inputs
    d.raw <- c()
    d.ens <- c()
    coord.t <- c()
    start <- 0
    ens <- TRUE
    if (grepl('wrf', run)){
        start <- 1
        ens <- FALSE
    }
    for (i in start:range){
        fcst <- sprintf("%03d", i)
        fname <- paste0(results.dir,'/',run,'/00/',fld,'/azmean_',fcst,'.dat')
        if (ens && file.exists(fname)){
            d.raw <-rbind(d.raw,read.table(fname))
            coord.t <- c(coord.t, i)
            d.mem <- c()
            for (m in 0:n.ens){
                mem <- sprintf("%02d", m)
                fname <- paste0(results.dir,'/',run,'/',mem,'/',fld,'/azmean_',fcst,'.dat')
                d.mem <- cbind(d.mem, data.manip(read.table(fname), fld.mult, fld.type))
            }
            d.ens <- rbind(d.ens, d.mem)
        } else {
            fname <- paste0(results.dir,'/',run,'/',fld,'/azmean_',fcst,'.dat')
            if (file.exists(fname)){
                d.raw <- rbind(d.raw,read.table(fname))
                coord.t <- c(coord.t, i)
            }
        }
    }
    series <- data.manip(d.raw, fld.mult, fld.type)
    valid <- series > fld.min
    if (length(d.ens) > 0){
        boot.mean <- function(x, i){return(mean(x[i]))}
        boot.do <- function(x){return(boot.ci(boot(x, boot.mean, R=1000),type="norm"))}
        d.ens.boot <- apply(d.ens, 1, boot.do)
        d.ens.range <- cbind(apply(d.ens, 1, min), apply(d.ens, 1, max))
        d.ens.mean <- c()
        d.ens.ci <- c()
        for (i in 1:length(d.ens.boot)){
            if (is.null(d.ens.boot[[i]])){
                d.ens.mean <- c(d.ens.mean, mean(d.ens[i,]))
                d.ens.ci <- rbind(d.ens.ci, rep(d.ens.mean[i], times=2))
            } else {
                d.ens.mean <- c(d.ens.mean, d.ens.boot[[i]]$t0)
                d.ens.ci <- rbind(d.ens.ci, c(d.ens.boot[[i]]$normal[2], d.ens.boot[[i]]$normal[3]))
            }
        }
        polygon(c(coord.t[valid], rev(coord.t[valid])),
                c(d.ens.range[,1][valid], rev(d.ens.range[,2][valid])),
                col=adjustcolor(cols[col.i], alpha.f=0.2), border=NA)
        polygon(c(coord.t[valid], rev(coord.t[valid])),
                c(d.ens.ci[,1][valid], rev(d.ens.ci[,2][valid])),
                col=adjustcolor(cols[col.i], alpha.f=0.6), border=NA)
#        lines(coord.t[valid], d.ens.mean[valid], col=cols[col.i], lwd=3, lty=1)
        lwd.ctl <- 2
    } else {
        lwd.ctl <- 3
    }

    # Generate plot
    lines(coord.t[valid], series[valid], col=cols[col.i], lwd=lwd.ctl)
}
runs.name <- sub('TC_idealized_520a19_', '', runs) 
legend(fld.leg.pos, inset=0.01, legend=c('WRF Reference', 'OFFB51', 'PHYWRF'), col=cols,
       cex=1.2, lwd=3)
par <- default
dev.off()
